#ifdef GL_ES
precision mediump float;
#endif

#ifndef saturate
#define saturate(v) clamp(v, 0.0, 1.0)
#endif

varying vec4 v_fragmentColor;
varying vec2 v_texCoord;

uniform int u_colorize;
uniform float u_h;
uniform float u_s;
uniform float u_l;

// reference: http://www.chilliant.com/rgb2hsv.html

vec3 rgb2hcv(vec3 c)
{
    vec4 k = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, k.wz), vec4(c.gb, k.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float C = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    float H = abs((q.w - q.y) / (6.0 * C + e) + q.z);
    return vec3(H, C, q.x);
}

vec3 rgb2hsl(vec3 c)
{
    vec3 hcv = rgb2hcv(c);
    float e = 1.0e-10;
    float L = hcv.z - hcv.y * 0.5;
    float S = hcv.y / (1.0 - abs(L * 2.0 - 1.0) + e);
    return vec3(hcv.x, S, L);
}

//vec3 rgb2hsv(vec3 c)
//{
//    vec3 hcv = rgb2hcv(c);
//    float e = 1.0e-10;
//    float S = hcv.y / (hcv.z + e);
//    return vec3(hcv.x, S, hcv.z);
//}

vec3 hue2rgb(float H)
{
    float R = abs(H * 6.0 - 3.0) - 1.0;
    float G = 2.0 - abs(H * 6.0 - 2.0);
    float B = 2.0 - abs(H * 6.0 - 4.0);
    return saturate(vec3(R, G, B));
}

vec3 hsl2rgb(vec3 hsl)
{
    vec3 rgb = hue2rgb(hsl.x);
    float C = (1.0 - abs(2.0 * hsl.z - 1.0)) * hsl.y;
    return saturate((rgb - 0.5) * C + hsl.z);
}

//vec3 hsv2rgb(vec3 c)
//{
//    vec4 k = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
//    vec3 p = abs(fract(c.xxx + k.xyz) * 6.0 - k.www);
//    return c.z * mix(k.xxx, saturate(p - k.xxx), c.y);
//}

void main()
{
    vec4 textureColor = v_fragmentColor * texture2D(CC_Texture0, v_texCoord);
    vec3 rgb = textureColor.rgb;
    if (textureColor.a > 0.0)
    {
        // Apply lightness
        rgb = saturate(rgb + mix(1.0 - rgb, rgb, step(u_l, 0.0)) * u_l / 100.0);

        if (rgb.r > 0.0 || rgb.g > 0.0 || rgb.b > 0.0)
        {
            vec3 hsl = rgb2hsl(rgb);
            if (u_colorize == 1)
            {
                hsl.x = u_h / 360.0;
                hsl.y = u_s / 100.0;
            }
            else
            {
                hsl.x = hsl.x + u_h / 360.0;
                hsl.y = hsl.y + u_s / 200.0;
            }

            hsl.x = mod(hsl.x, 1.0);
            hsl.y = saturate(hsl.y);
            rgb = hsl2rgb(hsl);
        }
    }

    gl_FragColor = vec4(rgb, textureColor.a);
}
